""" Cron job that retrieve weather data

To do
- Alarm when sensor not responding since xxx time.
- Add an option to deactivate server after some time when it is not responding.
"""

#mport pdb
import os
import sys
import transaction
import requests
import json

import logging

from datetime import datetime
from decimal import Decimal

import urllib3
urllib3.disable_warnings()

from meteocalc import Temp, dew_point, heat_index

from pyramid.paster import (
    get_appsettings,
    setup_logging,
)

from sqlalchemy import engine_from_config, func

import sqlalchemy.orm.exc

from sqlalchemy.orm import sessionmaker

from .models import (
    Base,
    Server,
    Sensor,
    Location,
    Compute,
    WeatherData,
    ToIgnore,
    SensorData,
)

class Cron(object):

    """This is Weather Data collecting service.

        It offer get method routed on /cron to be called in a cron task
        for example.
        Each time this method is called data is collected from
        different sensor servers.
        Each time this service is called the average value of all data read
        relative to each active location is saved. Therefore the cron task
        interval determine the data interval.
    """


    def __init__(self, settings, dbs):

        self.dbs = dbs

        self.logger = logging.getLogger(__name__)
        
        # Read configuration data from ini file
        self.intrv = int(settings['wd.interval']) * 60  # In seconds
        self.t_unit = settings['wd.t_unit']
        self.mailto = False
        if 'email.to' in settings:
            self.mailto = settings['email.to']
            self.mailfrm = settings['email.from']

        # Read key/value to identify sensors to ignore
        self.ign = {}
        lign = None
        itm = 0
        for ign in self.dbs.query(ToIgnore).order_by('gid'):
            if ign.gid not in self.ign:
                self.ign[ign.gid] = {}
            self.ign[ign.gid][ign.key] = ign.value



    def get(self):

        """This service read data from servers.
            For each server's URL the attached service is called and
            data are retrieved and processed.
        """

        for self.srv in self.dbs.query(Server).filter_by(
                active = True):

            # Read data from this server
            err = False
            try:
                rda = requests.get(self.srv.url, verify = False)
            except:
                err = str(sys.exc_info()[1])
            else:
                if rda.status_code != 200:
                    err = 'Return code ' + str(rda.status_code)
            if err is not False:
                self.logger.critical('wdwapp: cannot read server {srv} '
                    'because "{err}".\n'
                    'Server deactivated.'.format(
                        srv = self.srv.name, err = err))
                # deactivate this server
                self.srv.active = False
                self.dbs.commit()
            else:
                i1 = 0
                for line in rda.iter_lines():
                    i1 += 1
                    self.logger.debug("Process input line {}.".format(i1))
                    try:
                        self.dta = json.loads(line.decode('utf_8'))
                    except:
                        self.logger.error("Invalid json format of line :\n{line}".
                            format(line = line.decode('utf_8')))
                    self.trt_dta()
                    #self.dbs.flush()
                    self.dbs.commit()
                    #transaction.commit()


    def trt_dta(self):

        """This function process one data.
        
            First a corresponded sensor is searched.
            If found and if sensors is attached to a location,
            data is added to this location.
            Otherwise data is backed up.
            If no sensor match is found a new sensor is created
            with new flag on.
        """

        # Filter data that are ignored
        # Data is compared against the to_ignore table.
        # If the data contain all key/value with the same gid,
        # this data is skipped.
        for gid in self.ign:
            found = True
            for key, val in self.ign[gid].items():
                if key not in self.dta or self.dta[key] != val:
                    found = False
                    break
            if found:
                return

        # Time and model are mandatory
        if 'time' not in self.dta:
            self.logger.error("Entry without 'time' : " + str(self.dta))
            return
        if 'model' not in self.dta:
            self.logger.error("Entry without 'model' : " + str(self.dta))
            return
        model = self.dta['model']

        # default for non mandatory data
        devid = None
        channel = 0

        # Read timestamp of data 
        self.moment = datetime.strptime(self.dta['time'], '%Y-%m-%d %H:%M:%S')
            
        # Query sensors
        sens = self.dbs.query(Sensor).filter_by(model = model)
        if 'id' in self.dta:
            devid = self.dta['id']
            sens = sens.filter_by(devid = devid)
        if 'channel' in self.dta:
            channel = int(self.dta['channel'])
            sens = sens.filter_by(channel = channel)

        # Is sensor known ?
        try:
            self.sens = sens.one()
            
        except sqlalchemy.orm.exc.NoResultFound:
            
            self.logger.debug('Sensor ' + str(self.dta) + ' Not found')

            # Add new sensor
            cnt = self.dbs.query(Sensor).count()
            while True:  # Loop until new sensor found
                name = 'sensor'+str(cnt + 1)
                if self.dbs.query(Sensor).filter_by(
                        name = name).one_or_none() is None:
                    break
                cnt += 1
            self.dbs.add(Sensor(sid = self.srv.id,
                name = name,
                model = model,
                devid = devid,
                channel = channel,
                first_seen = self.moment,
                last_seen = self.moment,
                nbr_seen = 1,
                new = True))
            self.logger.warning("New sensor '" + name + "' found and added.\n" +
                'Data : ' + str(self.dta))
                    
            # Backup first data for later use
            sens = self.dbs.query(func.max(Sensor.id))
            self.dbs.add(SensorData(sid = sens[0][0],
                timestp = self.moment,
                data = str(self.dta)))

        except sqlalchemy.orm.exc.MultipleResultsFound:
            
            self.logger.error('Sensor ' + str(self.dta) + ' Found multiple time')
            
        else:

            # Update last time and number of seen
            self.sens.last_seen = self.moment
            self.sens.nbr_seen += 1

            # Find if sensor is attached to a location
            try:
                self.loc = self.dbs.query(Location).filter_by(sid = self.sens.id).one()
                
            except sqlalchemy.orm.exc.NoResultFound:

                # Backup this data if no location
                pass
                
            else:

                # If is inactive, compute avarage
                if self.loc.active:
                    self.compute()
                    return

            # Backup this data for later use
            self.dbs.add(SensorData(sid = self.sens.id,
                timestp = self.moment,
                data = str(self.dta)))

    def compute(self):

        """Compute sensor data read.

            Following capacity of sensor, data is added and counted.
            Battery level is also checked and a message is sent the
            first time level is LOW.
            Data available here :
                self.dta : Data read
                self.sens : sensor
                self.loc : Location
        """

        # Units ==============================================================
        self.tunit = '°' + self.t_unit.upper();

        # Battery level ======================================================

        # Read battery level if available
        if self.sens.cap_bat:
            bat = self.sens.cap_bat.split('|')
            if self.dta[bat[0]] == bat[1]:

                # If OK update sensor
                if self.sens.bat_low:
                    self.logger.warning('Warning: At {tmp}, battery level from sensor '
                        '"{sensor}" in location "{loc}" is again OK.'.format(
                            tmp = self.moment,
                            sensor = self.sens.name,
                            loc = self.loc.name))
                    self.sens.bat_low = False
                    
            else:

                # If it is the first time battery is not OK, send message
                if not self.sens.bat_low:
                    self.logger.critical('Warning: At {tmp}, battery level from sensor '
                        '"{sensor}" in location "{loc}" is LOW.'.format(
                            tmp = self.moment,
                            sensor = self.sens.name,
                            loc = self.loc.name))
                    self.sens.bat_low = True

        # Compute data and send alarms =======================================
        tmst = self.moment.timestamp()  # Convert to Unix time stamp

        # Get last reading for the current location
        try:
            self.cmpt = self.dbs.query(Compute).filter_by(lid = self.loc.id).one()

        except sqlalchemy.orm.exc.NoResultFound:

            # If not found, create a new one
            t1 = tmst - (tmst % self.intrv)     # Previous interval
            self.dbs.add(Compute(
                lid = self.loc.id,
                start = t1,
                last  = t1 + self.intrv,
                t_sum = 0, t_count = 0,
                h_sum = 0, h_count = 0,
                p_sum = 0, p_count = 0,
                t_min = False, t_max = False))
            
            # Get the new compute just added
            self.cmpt = self.dbs.query(Compute).filter_by(lid = self.loc.id).one()

        # Compute values
        if self.sens.cap_t:  # Temperature
            temp = Decimal(self.dta[self.sens.cap_t])
            self.cmpt.t_sum   += round(temp, 2)
            self.cmpt.t_count += 1

        if self.sens.cap_h:  # Humidity
            h = round(self.dta[self.sens.cap_h], 2)
            if h < 1:
                h = 0
            if h > 100:
                h = 100
            self.cmpt.h_sum   += h
            self.cmpt.h_count += 1

        if self.sens.cap_p:  # Pressure
            self.cmpt.p_sum   += self.dta[self.sens.cap_p]
            self.cmpt.p_count += 1

        # Save data if time interval reached =================================
        
        # if time limit has been reached
        if tmst >= self.cmpt.last:

            # save weather data
            if self.cmpt.h_count > 0:
                H = float(self.cmpt.h_sum / self.cmpt.h_count)
            else:
                H = 0.0
            if self.cmpt.t_count > 0:
                temp0 = self.cmpt.t_sum / self.cmpt.t_count
            else:
                temp0 = 0
            T0 = Temp(temp0, self.sens.t_unit)
            TD = dew_point(temperature=T0, humidity=H)
            TF = heat_index(temperature=T0, humidity=H)
            if self.t_unit == 'c':
                T = T0.c
                TD = TD.c
                TF = TF.c
            elif self.t_unit == 'f':
                T = T0.f
                TD = TD.f
                TF = TF.f
            else:
                T = T0.k
                TD = TD.k
                TF = TF.k
            if self.cmpt.p_count > 0:
                P = self.cmpt.p_sum / self.cmpt.p_count
            else:
                P = 0
            timestp = datetime.fromtimestamp(self.cmpt.last)
            try:
                self.dbs.query(WeatherData).filter_by(timestp = timestp, lid = self.loc.id).one()
            except sqlalchemy.orm.exc.NoResultFound:
                self.dbs.add(WeatherData(
                    timestp = timestp,
                    lid = self.loc.id,
                    temperature = round(T, 2),
                    humidity = H,
                    pressure = round(P, 2),
                    t_dew = round(TD, 2),
                    t_feel = round(TF, 2)))
                self.trigger_alarm(timestp, temp = temp0)

            # Clear compute data
            self.cmpt.start = self.cmpt.last
            self.cmpt.last += self.intrv
            self.cmpt.t_sum = 0
            self.cmpt.t_count = 0
            self.cmpt.h_sum = 0
            self.cmpt.h_count = 0
            self.cmpt.p_sum = 0
            self.cmpt.p_count = 0


    def trigger_alarm(self, timestp, temp=None):

        """Trigger alarm for temperature, pressure or humidity

            if temp is passed, trigger temperature alarm
        """

        # Temperature alarm
        if temp is not None:

            # Minimum alarm
            if self.loc.t_min is not None:
                if temp < self.loc.t_min:

                    # Temperature below minimum allowed ==> Alarm
                    if not self.cmpt.t_min:
                        self.logger.critical('Warning: At {tmp}, temperature in location "{loc}" ' 
                        'has passed below the minimum of {temp} {unit}.'.format(
                            tmp = timestp,
                            loc = self.loc.name,
                            temp = self.loc.t_min,
                            unit = self.tunit))
                        self.cmpt.t_min = True   # Alarm just one time
                else:
                    # Reset previously alarm
                    if self.cmpt.t_min:
                        self.logger.warning('Warning: At {tmp}, temperature in location "{loc}" ' 
                        'is again over the minimum of {temp}  {unit}.'.format(
                            tmp = timestp,
                            loc = self.loc.name,
                            temp = self.loc.t_min,
                            unit = self.tunit))
                        self.cmpt.t_min = False

            # Maximum alarm
            if self.loc.t_max is not None:
                if temp > self.loc.t_max:

                    # Temperature over maximum allowed ==> Alarm
                    if not self.cmpt.t_max:
                        self.logger.critical('Warning: At {tmp}, temperature in location "{loc}" ' 
                        'has exceeded the maximum of {temp}  {unit}.'.format(
                            tmp = timestp,
                            loc = self.loc.name,
                            temp = self.loc.t_max,
                            unit = self.tunit))
                        self.cmpt.t_max = True   # Alarm just one time
                else:
                    # Reset previously alarm
                    if self.cmpt.t_max:
                        self.logger.warning('Warning: At {tmp}, temperature in location "{loc}" ' 
                        'is again under the maximum of {temp}  {unit}.'.format(
                            tmp = timestp,
                            loc = self.loc.name,
                            temp = self.loc.t_max,
                            unit = self.tunit))
                        self.cmpt.t_max = False


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri>\n'
          '(example: "%s development.ini")' % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) != 2:
        usage(argv)
    config_uri = argv[1]
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    engine = engine_from_config(settings, 'sqlalchemy.')
    with engine.connect() as conn:
        DBSession = sessionmaker(bind=conn)()
        Base.metadata.create_all(engine)

        cron = Cron(settings, DBSession)
        cron.get()

        DBSession.close()



import logging
import datetime
import requests
import sys


class SMSHandler(logging.Handler):


    def __init__(self, arg_url, arg_user, arg_pwd, arg_msg):

        """
        Initializes the SMSHandler
        """

        logging.Handler.__init__(self)
        self.url  = arg_url
        self.user = arg_user.split('=')
        self.pwd  = arg_pwd.split('=')
        self.msg  = arg_msg

        self.active = True

        self.logger = logging.getLogger(__name__)
        

    def emit(self, record):

        """
        Sends the message
        """

        # Log if service inactive
        if not self.active:
            self.logger.warning('SMS service is inactive')
            return

        # Compose message
        to_send = self._construct_message(record)

        # Send message
        err = False
        try:
            r = requests.get(self.url, params = {
                        self.user[0]: self.user[1],
                        self.pwd[0] : self.pwd[1],
                        self.msg: to_send})
        except:
            err = str(sys.exc_info()[1])
        else:
            if r.status_code != 200:
                if  r.status_code == 400:
                    err = 'missing parameter.'
                elif r.status_code == 402:
                    err = 'To many SMS sent in a to short time.'
                elif r.status_code == 403:
                    err = 'Service not activated or invalid key.'
                elif r.status_code == 500:
                    err = 'Server error. Try later.'
                else:
                    err = 'Unknown error : ' + str(r.status_code)
        if err is not False:
            self.active = False     # To not enter in a infinite loop
            self.logger.error('Cannot send SMS : ' + err)


    def _construct_message(self, record):

        """
        Contruct and format the mesage to be sent.

        i.e
        MODULE: sms_log_handler.sms_handler

        LEVEL: ERROR

        TIME: 21, May 2017 10:54

        MESSAGE: the message from logging call
        """

        msg = (
            'MODULE: {module_path}\nLEVEL: {level}\nTIME: {time}\n'
            'MESSAGE:\n{msg}')
        date_time = datetime.datetime.fromtimestamp(record.created)
        date_time = date_time.strftime('%d, %b %Y %H:%M')
        formatted_msg = msg.format(
            level=record.levelname, time=date_time, msg=record.getMessage(),
            module_path=record.name, line_no=record.lineno)
        return formatted_msg



from pyramid.security import Allow, Everyone

from sqlalchemy import (
    Column,
    ForeignKey,
    Integer,
    BigInteger,
    String,
    DateTime,
    Numeric,
    Text,
    Boolean,
    CheckConstraint,
)

from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.orm import (
    scoped_session,
    sessionmaker,
)

# Le 3.10.2020 selon https://pypi.org/project/zope.sqlalchemy/#id3
# from zope.sqlalchemy import ZopeTransactionExtension
# DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
from zope.sqlalchemy import register
DBSession = scoped_session(sessionmaker())
register(DBSession)
Base = declarative_base()


class Server(Base):

    "Table of servers who deliver data"

    __tablename__   = 'server'
    
    id              = Column(Integer, primary_key=True,
                        doc="Server unique ID")
    name            = Column(String(20), unique=True,
                        doc="Server unique name")
    url             = Column(Text,
                        doc="URL where to get data")
    long_desc       = Column(Text,
                        doc="Description")
    active          = Column(Boolean,
                        doc="Is this server active (0/1)")


class Sensor(Base):

    "Table of sensors"

    __tablename__   = 'sensor'

    # Identification
    id              = Column(Integer, primary_key=True,
                        doc="Sensor unique ID")
    sid             = Column(Integer, ForeignKey('server.id'),
                        doc="ID of server from where this sensor's data came")
    name            = Column(String(20), unique=True,
                        doc="Sensor unique name")
    short_desc      = Column(String(100),
                        doc="Text displayed in lists, on graphs, etc.")
    long_desc       = Column(Text,
                        doc="Description containing some useful things")
    model           = Column(String(100),
                        doc="Sensor model")
    devid           = Column(String(100),
                        doc="Sensor device ID")
    channel         = Column(Integer,
                        doc="Sensor channel")

    # Operational
    first_seen      = Column(DateTime,
                        doc="Creation time of this sensor")
    last_seen       = Column(DateTime,
                        doc="Last time receiving data from this sensor")
    nbr_seen        = Column(Integer,
                        doc="Number of time receiving data from this sensor")
    new             = Column(Boolean,
                        doc="This sensor has not yet be seen (0/1)")
    cap_bat         = Column(String(100),
                        doc="This sensor give battery level (None or Id string|OK string)")
    bat_low         = Column(Boolean,
                        doc="Battery level LOW (0/1)")
    last_bat_chg    = Column(DateTime,
                        doc="Last time battery was changed for this sensor")
    nbr_bat_chg     = Column(Integer,
                        doc="Number of time battery was changed for this sensor")

    # Capabilities
    cap_h           = Column(String(100),
                        doc="This sensor can read humidity (None or Id string)")
    cap_t           = Column(String(100),
                        doc="This sensor can read temperature (None or Id string)")
    t_unit          = Column(String(1), CheckConstraint("t_unit in ('c','f','k')"), default='c',
                        doc="Temperature unit of sensor (c=celsius, f=fahrenheit, k=kelvin")
    cap_p           = Column(String(100),
                        doc="This sensor can read pressure (None or Id string)")
    p_unit          = Column(String(1), CheckConstraint("p_unit in ('p','b','m')"), default='p',
                        doc="Pressure unit of sensor (p=hectopascal, b=millibar, m=millimeter of mercury")


class Location(Base):

    "Table of locations"

    __tablename__   = 'location'
    
    id              = Column(Integer, primary_key=True,
                        doc="Location unique ID")
    rank              = Column(Integer, index=True,
                        doc="Sort rank")
    name            = Column(String(20), unique=True,
                        doc="Location unique name")
    short_desc      = Column(String(100),
                        doc="Text displayed in lists, on graphs, etc.")
    long_desc       = Column(Text,
                        doc="Description containing some useful things")
    sid             = Column(Integer, ForeignKey('sensor.id'),
                        unique=True,
                        doc="ID of sensor attached to this location")
    t_min           = Column(Numeric(5, 2),
                        doc="Temperature below which alarm is triggered (or None)")
    t_max           = Column(Numeric(5, 2),
                        doc="Temperature above which alarm is triggered (or None)")
    on_graph        = Column(Boolean,
                        doc="Is this location on graph (0/1)")
    gr_color        = Column(String(6),
                        doc="Color on graph")
    active          = Column(Boolean,
                        doc="Is this location active (0/1)")


class WeatherData(Base):

    "Table of weather data incoming from sensors"

    __tablename__   = 'weather_data'
    
    timestp         = Column(DateTime, primary_key=True,
                        doc="Date time of reading")
    lid             = Column(Integer, ForeignKey('location.id'),
                        primary_key=True,
                        doc="ID in location table")
    temperature     = Column(Numeric(5, 2),
                        doc="Temperature been read")
    humidity        = Column(Integer,
                        doc="Humidity been read")
    pressure        = Column(Numeric(5, 2),
                        doc="Pressure been read")
    t_dew           = Column(Numeric(5, 2),
                        doc="Dew point calculated from temp & humidity")
    t_feel          = Column(Numeric(5, 2),
                        doc="feels like temp. calculated from temp. & humidity")


class Compute(Base):

    "Table of temporary weather data used to compute average values"

    __tablename__   = 'compute'
    
    lid             = Column(Integer, ForeignKey('location.id'),
                        primary_key=True,
                        doc="ID in location table")
    start           = Column(BigInteger,
                        doc="Timestanp of first reading")
    last            = Column(BigInteger,
                        doc="Timestanp of last reading")
    t_sum           = Column(Numeric(10, 2),
                        doc="Summation of temperature since start")
    t_count         = Column(Integer,
                        doc="Number of temperature since start")
    h_sum           = Column(Numeric(10, 2),
                        doc="Summation of humidity since start")
    h_count         = Column(Integer,
                        doc="Number of humidity since start")
    p_sum           = Column(Integer,
                        doc="Summation of pressure since start")
    p_count         = Column(Integer,
                        doc="Number of pressure since start")
    t_min           = Column(Boolean,
                        doc="Minimal temp alarm is triggered")
    t_max           = Column(Boolean,
                        doc="Maximal temp alarm is triggered")


class ToIgnore(Base):

    """Table of sensors to ignore
        Data that match all keys/value with same gid are ignored.
    """

    __tablename__   = 'to_ignore'

    # Identification
    id              = Column(Integer, primary_key=True,
                        doc="Sensor unique ID")
    gid             = Column(Integer, index=True,
                        doc="Sensor ignore group ID")
    key             = Column(String(100),
                        doc="Key to match")
    value           = Column(String(100),
                        doc="Value to match")

class SensorData(Base):

    """New sensor Data
        Store unrecognized data (to purge regularly)
        Can be used, for example, to reload data from new sensor after
        it is activated or associated to a location.
    """

    __tablename__   = 'sensor_data'
    
    id              = Column(Integer, primary_key=True,
                        doc="Location unique ID")
    sid             = Column(Integer, ForeignKey('sensor.id'),
                        index=True,
                        doc="ID of sensor attached to this location")
    timestp         = Column(DateTime, index=True,
                        doc="Date time of reading")
    data            = Column(Text,
                        doc="Data as read from sensor")


class Root(object):
    __acl__ = [(Allow, Everyone, 'view'),
                (Allow, 'group:editors', 'edit')]

    def __init__(self, request):
        pass



""" Main programm 

Show wheater data and mamange configuration

To do
- Add message to send with alarm
- Add authentication to allow some users to change values and/or
  configuration data.
- Added auth.secret in ini files. Please merge your ini files with them on
  http://static.frkb.fr/wdwapp
- Add .ini parameter to define period of detail view.
"""
#import pdb
import colander
import deform.widget

from wdwapp import __version__, __year__

from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config

from pyramid.httpexceptions import HTTPFound
from pyramid.security import (
    remember,
    forget,
)

from .models import (
    DBSession,
    Location,
    Sensor,
    WeatherData,
)

from sqlalchemy import desc, func

from datetime import datetime, timedelta

class WikiPage(colander.MappingSchema):
    title = colander.SchemaNode(colander.String())
    body = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.RichTextWidget()
    )

class WeatherViews(object):
    def __init__(self, request):
        self.request = request
        self.mode = ''
        self.version = __version__
        self.year = __year__
        self.stg = self.request.registry.settings
        self.t_unit = self.stg['wd.t_unit'].upper()
        minutes = int(self.stg['wd.max_data_age'])
        self.max_age = timedelta(minutes=minutes)


    @property
    def wiki_form(self):
        schema = WikiPage()
        btns=['submit',]
        if self.mode == 'e':
            btns.append('delete')
        return deform.Form(schema, buttons=btns)


    @property
    def reqts(self):
        return self.wiki_form.get_widget_resources()


    @view_config(route_name='overview', renderer='templates/overview.pt')
    def overview(self):

        """Main view with an overview of locations
        """

        # Load session data
        session = self.request.session

        # Get last reading time
        lt = DBSession.query(func.max(WeatherData.timestp))
        #pdb.set_trace()
        ltime = lt[0][0].strftime('%d %b à %H:%M')
        
        # Calculate max oldest data
        
        
        # Retrieve and read last data for active locations
        datas = []
        for loc in DBSession.query(Location).filter_by(active = True).order_by('rank'):
            
            # Read location data
            lt = DBSession.query(func.max(WeatherData.timestp)).filter_by(lid = loc.id)[0][0]
            wdt = DBSession.query(WeatherData).filter_by(lid = loc.id, timestp = lt)[0]
            sns = DBSession.query(Sensor).filter_by(id = loc.sid)[0]
            
            # Decode battery level
            if not sns.bat_low:
                batlevel = "OK"
            else:
                batlevel = "KO"
            
            # Verify if data is up to date
            if sns.last_seen < datetime.now() - self.max_age:
                uptodate = 'KO'
            else:
                uptodate = 'OK'
            
            
            # Fill template data
            datas.append({'id': loc.id,
                'name': loc.name,
                'temperature': wdt.temperature,
                'humidity': wdt.humidity,
                'tunit': '°' + self.t_unit,
                'bat_level': batlevel,
                'uptodate': uptodate })
        #pdb.set_trace()

        return dict(ltime=ltime, datas=datas)

    
    @view_config(route_name='detail', renderer='templates/detail.pt')
    def detail(self):

        """Detail of a location.
            Display the list of last 24h.
        """
        
        lid = self.request.matchdict['lid']
        lname = DBSession.query(Location.name, Location.sid).filter_by(id = lid)[0][0]
        datas = DBSession.query(WeatherData).\
                filter_by(lid = lid).\
                filter(func.MINUTE(WeatherData.timestp) == 0).\
                order_by(desc('timestp')).limit(96).all()
        return dict(lname = lname, datas = datas,
            tunit = '°' + self.t_unit)


    @view_config(route_name='wikipage_add',
                 renderer='wikipage_addedit.pt')
    def wikipage_add(self):
        self.mode = 'a'
        form = self.wiki_form.render()

        if 'submit' in self.request.params:
            controls = self.request.POST.items()
            try:
                appstruct = self.wiki_form.validate(controls)
            except deform.ValidationFailure as e:
                # Form is NOT valid
                return dict(form=e.render())

            # Form is valid, Add a new page to the database
            new_title = appstruct['title']
            new_body = appstruct['body']
            DBSession.add(Page(title=new_title, body=new_body))

            # Get the new ID and redirect
            page = DBSession.query(Page).filter_by(title=new_title).one()
            new_uid = page.uid

            # Now visit new page
            url = self.request.route_url('wikipage_view', uid=new_uid)
            return HTTPFound(url)

        return dict(form=form)

    @view_config(route_name='wikipage_edit',
                 renderer='wikipage_addedit.pt')
    def wikipage_edit(self):
        uid = self.request.matchdict['uid']
        page = DBSession.query(Page).filter_by(uid=uid).one()

        self.mode = 'e'
        wiki_form = self.wiki_form

        if 'submit' in self.request.params:
            controls = self.request.POST.items()
            try:
                appstruct = wiki_form.validate(controls)
            except deform.ValidationFailure as e:
                return dict(page=page, form=e.render())

            # Change the content and redirect to the view
            page.title = appstruct['title']
            page.body  = appstruct['body']

            url = self.request.route_url('wikipage_view', uid=uid)
            return HTTPFound(url)
        
        if 'delete' in self.request.params:
            session = self.request.session
            session['msg'] = 'Page "' + page.title + '" deleted.'
            DBSession.delete(page)

            url = self.request.route_url('wiki_view')
            return HTTPFound(url)

        form = wiki_form.render(dict(
            uid=page.uid, title=page.title, body=page.body)
        )

        return dict(page=page, form=form)

    @view_config(route_name='home')
    def home(self):
        first = self.request.matchdict['first']
        last = self.request.matchdict['last']
        return {
            'name': 'Home View',
            'first': first,
            'last': last
        }




import os
import sys
import transaction

from sqlalchemy import engine_from_config

from pyramid.paster import (
    get_appsettings,
    setup_logging,
)

from .models import (
    DBSession,
    Base,
)


def usage(argv):
    cmd = os.path.basename(argv[0])
    print('usage: %s <config_uri>\n'
          '(example: "%s development.ini")' % (cmd, cmd))
    sys.exit(1)


def main(argv=sys.argv):
    if len(argv) != 2:
        usage(argv)
    config_uri = argv[1]
    setup_logging(config_uri)
    settings = get_appsettings(config_uri)
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.create_all(engine)



__version__ = '0.1.3'
__year__ = 2021

def main(global_config, **settings):

    from pyramid.config import Configurator
    from pyramid.session import SignedCookieSessionFactory

    from pyramid.authentication import AuthTktAuthenticationPolicy
    from pyramid.authorization import ACLAuthorizationPolicy
    from .security import groupfinder

    from sqlalchemy import engine_from_config
    from .models import DBSession, Base, Root

    #engine = engine_from_config(settings, 'sqlalchemy.', pool_recycle=3600)
    engine = engine_from_config(settings, 'sqlalchemy.', pool_pre_ping=True) # From V 1.2
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine

    my_session_factory = SignedCookieSessionFactory(
        settings['session.secret'])

    config = Configurator(settings=settings,
                    session_factory=my_session_factory,
                    root_factory='wdwapp.models.Root')

    config.include('pyramid_chameleon')

    # Security policies
    authn_policy = AuthTktAuthenticationPolicy(
        settings['auth.secret'], callback=groupfinder,
        hashalg='sha512')
    authz_policy = ACLAuthorizationPolicy()
    config.set_authentication_policy(authn_policy)
    config.set_authorization_policy(authz_policy)
    
    # Routes for web site
    config.add_route('overview', '/')
    config.add_route('detail', '/detail/{lid}')
    config.add_route('wikipage_add', '/add')
    config.add_route('wikipage_edit', '/{uid}/edit')
    config.add_route('home', '/howdy/{first}/{last}')

    # Static things
    config.add_static_view('static', 'static', cache_max_age=3600)
    config.add_static_view('deform_static', 'deform:static/')

    # Let's go
    config.scan('.views')
    return config.make_wsgi_app()



"""Version 0.1.0

Revision ID: baa78950306e
Revises: 
Create Date: 2021-10-14 20:28:53.875007

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'baa78950306e'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(op.f('ix_location_rank'), 'location', ['rank'], unique=False)
    op.create_unique_constraint(None, 'location', ['sid'])
    op.add_column('sensor', sa.Column('last_bat_chg', sa.DateTime(), nullable=True))
    op.add_column('sensor', sa.Column('nbr_bat_chg', sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('sensor', 'nbr_bat_chg')
    op.drop_column('sensor', 'last_bat_chg')
    op.drop_constraint(None, 'location', type_='unique')
    op.drop_index(op.f('ix_location_rank'), table_name='location')
    # ### end Alembic commands ###



import os
#import pdb

from setuptools import setup, find_packages
from glob import glob

from wdwapp import __version__

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.rst')) as f:
    README = f.read()
with open(os.path.join(here, 'CHANGES.rst')) as f:
    CHANGES = f.read()

requires = [
    'pyramid',
    'pyramid_chameleon',
    'deform',
    'sqlalchemy',
    'pyramid_tm',
    'gunicorn',
    'zope.sqlalchemy',
    'mysqlclient',
    'meteocalc',
    'requests',
    'bcrypt',
    'alembic',
]
#pdb.set_trace()

setup(name = 'wdwapp',
    version = __version__,
    description = 'Webapps for recording and displaying weather data',
    long_description = README + '\n\n' + CHANGES,
    long_description_content_type = 'text/x-rst',
    author = 'Frédéric KIEBER',
    author_email = 'contact@frkb.fr',
    license = 'MIT',
    url = 'https://github.com/fkieber/wdwapp',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Framework :: Pyramid',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'Topic :: Scientific/Engineering :: Atmospheric Science',
    ],
    keywords = 'webapp wdwsrv weather',
    packages = find_packages(),
    include_package_data = True,
    zip_safe = False,
    install_requires = requires,
    entry_points = {
        'paste.app_factory': [
            'main = wdwapp:main',   # main from __init__.py
        ],
        'console_scripts' : [
            'wdwapp_initialize_db = wdwapp.initialize_db:main',
            'wdwapp_cron = wdwapp.cron:main',
        ],
    },
    #data_files = [
	#	('alembic', glob('wdwapp/alembic/**/*', recursive=True)),
	#],
)

